<?php
/*
 * This file is part of the Manualranking Product plugin
 *
 * Copyright (C) 2016 LOCKON CO.,LTD. All Rights Reserved.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Plugin\Manualranking\ServiceProvider;

use Plugin\Manualranking\Form\Type\ManualrankingProductType;
use Plugin\Manualranking\Service\ManualrankingService;
use Plugin\Manualranking\Utils\Version;
use Silex\Application as BaseApplication;
use Silex\Application;
use Silex\ServiceProviderInterface;
use Symfony\Component\Translation\Translator;
use Symfony\Component\Yaml\Yaml;

// include log functions (for 3.0.0 - 3.0.11)
require_once(__DIR__.'/../log.php');

/**
 * Class ManualrankingServiceProvider.
 */
class ManualrankingServiceProvider implements ServiceProviderInterface
{
    /**
     * @param BaseApplication $app
     */
    public function register(BaseApplication $app)
    {
        // 手動売上ランキング情報テーブルリポジトリ
        $app['eccube.plugin.manualranking.repository.manualranking_product'] = $app->share(function () use ($app) {
            return $app['orm.em']->getRepository('Plugin\Manualranking\Entity\ManualrankingProduct');
        });

        // 手動売上ランキングの一覧
        $app->match('/'.$app['config']['admin_route'].'/manualranking', '\Plugin\Manualranking\Controller\ManualrankingController::index')
            ->bind('admin_manualranking_list');

        // 手動売上ランキングの新規先
        $app->match('/'.$app['config']['admin_route'].'/manualranking/new', '\Plugin\Manualranking\Controller\ManualrankingController::edit')
            ->value('id', null)->assert('id', '\d+|')
            ->bind('admin_manualranking_new');

        // 手動売上ランキングの編集
        $app->match('/'.$app['config']['admin_route'].'/manualranking/{id}/edit', '\Plugin\Manualranking\Controller\ManualrankingController::edit')
            ->value('id', null)->assert('id', '\d+|')
            ->bind('admin_manualranking_edit');

        // 手動売上ランキングの削除
        $app->delete('/'.$app['config']['admin_route'].'/manualranking/{id}/delete', '\Plugin\Manualranking\Controller\ManualrankingController::delete')
        ->value('id', null)->assert('id', '\d+|')
        ->bind('admin_manualranking_delete');

        // move rank
        $app->post('/'.$app['config']['admin_route'].'/manualranking/rank/move', '\Plugin\Manualranking\Controller\ManualrankingController::moveRank')
            ->bind('admin_manualranking_rank_move');

        // 商品検索画面表示
        $app->post('/'.$app['config']['admin_route'].'/manualranking/search/product', '\Plugin\Manualranking\Controller\ManualrankingSearchModelController::searchProduct')
            ->bind('admin_manualranking_search_product');

        $app->match('/'.$app['config']['admin_route'].'/manualranking/search/product/page/{page_no}', '\Plugin\Manualranking\Controller\ManualrankingSearchModelController::searchProduct')
            ->assert('page_no', '\d+')
            ->bind('admin_manualranking_search_product_page');

        // ブロック
        $app->match('/block/manualranking_product_block', '\Plugin\Manualranking\Controller\Block\ManualrankingController::index')
            ->bind('block_manualranking_product_block');

        // 型登録
        $app['form.types'] = $app->share($app->extend('form.types', function ($types) use ($app) {
            $types[] = new ManualrankingProductType($app);

            return $types;
        }));

        // サービスの登録
        $app['eccube.plugin.manualranking.service.manualranking'] = $app->share(function () use ($app) {
            return new ManualrankingService($app);
        });

        // メッセージ登録
        $app['translator'] = $app->share($app->extend('translator', function (Translator $translator, Application $app) {
            $file = __DIR__.'/../Resource/locale/message.'.$app['locale'].'.yml';
            if (file_exists($file)) {
                $translator->addResource('yaml', $file, $app['locale']);
            }

            return $translator;
        }));

        // Config
        $app['config'] = $app->share($app->extend('config', function ($config) {
            // menu bar
            $addNavi['id'] = 'admin_manualranking';
            $addNavi['name'] = '手動売上ランキング管理';
            $addNavi['url'] = 'admin_manualranking_list';
            $nav = $config['nav'];
            foreach ($nav as $key => $val) {
                if ('content' == $val['id']) {
                    $nav[$key]['child'][] = $addNavi;
                }
            }
            $config['nav'] = $nav;

            // Update path
            $pathFile = __DIR__.'/../Resource/config/path.yml';
            if (file_exists($pathFile)) {
                $path = Yaml::parse(file_get_contents($pathFile));
                if (!empty($path)) {
                    // Replace path
                    $config = array_replace_recursive($config, $path);
                }
            }

            // Update constants
            $constantFile = __DIR__.'/../Resource/config/constant.yml';
            if (file_exists($constantFile)) {
                $constant = Yaml::parse(file_get_contents($constantFile));
                if (!empty($constant)) {
                    // Replace constants
                    $config = array_replace_recursive($config, $constant);
                }
            }

            return $config;
        }));

        // initialize logger (for 3.0.0 - 3.0.8)
        if (!Version::isSupportGetInstanceFunction()) {
            eccube_log_init($app);
        }
    }

    /**
     * @param BaseApplication $app
     */
    public function boot(BaseApplication $app)
    {
    }
}
